/**
** This software is distributed under the terms of the General Public License.  
** 
** Program : BuildIT!
** Version : 0.2
** Authors : Artur Prusakiewicz <prusak@mat.uni.torun.pl>
**	     Michal Switala <scisly@mat.uni.torun.pl>
** E-MAIL  : build@mat.uni.torun.pl
** WWW     : http://www.mat.uni.torun.pl/~scisly/buildit/
**           http://www.mat.uni.torun.pl/~prusak/buildit/
**/

#include "MainWindow.h"
#include "comments.h"

void press(QWidget *widget,QMouseEvent *event){}
void mv(QWidget *widget,QMouseEvent *event){}
void release(QWidget *widget,QMouseEvent *event){}
void dbclick(QWidget *widget,QMouseEvent *event){}
void itsMe(MyClass *widget){}

MainWindow::MainWindow(QWidget *parent, const char *name ): QMainWindow( parent, name )
{
  int i;

  prepare();
  setDockEnabled(QMainWindow::Top,TRUE);
  setRightJustification(TRUE);
  setUsesBigPixmaps(TRUE);

  middle=new Dialog(this,"dialog");
  setToolBar();

  file = new QPopupMenu();
  CHECK_PTR( file );
  file->insertItem("Ne&w", this, SLOT(newOne()), CTRL+Key_N );
  file->insertItem("&Open...",  this, SLOT(open()), CTRL+Key_O );
  saveItem=file->insertItem("&Save", this, SLOT(save()), CTRL+Key_S );
  saveAsItem=file->insertItem("S&ave As...",this,SLOT(saveAs()),CTRL+Key_A); 
  allowSave=FALSE;
  closeItem=file->insertItem( "&Close", this, SLOT(closeIt()), CTRL+Key_W );
  allowClose=FALSE;
  file->insertSeparator();

  file->insertSeparator();
  file->insertItem( "E&xit", this, SLOT(exit()), CTRL+Key_Q );
  file->setItemEnabled(saveItem,FALSE);
  file->setItemEnabled(saveAsItem,FALSE);
  file->setItemEnabled(closeItem,FALSE);

  edit = new QPopupMenu();
  CHECK_PTR( edit );
  undoItem=edit->insertItem( "&Undo", this, SLOT(undo()),CTRL+Key_Z );
  allowUndo=FALSE;

  edit->insertSeparator();

  cutItem=edit->insertItem("&Cut",this,SLOT(cut()),CTRL+Key_X);
  edit->setItemEnabled(cutItem,FALSE);
  allowCut=FALSE;

  copyItem=edit->insertItem("C&opy",this,SLOT(copy()),CTRL+Key_C);
  edit->setItemEnabled(copyItem,FALSE);
  allowCopy=FALSE;

  pasteItem=edit->insertItem("&Paste",this,SLOT(paste()),CTRL+Key_V);
  edit->setItemEnabled(undoItem,FALSE);
  allowPaste=FALSE;

  edit->setItemEnabled(pasteItem,FALSE);

  edit->insertSeparator();

  prefsItem=edit->insertItem("P&roperties",this,SLOT(prefs()),CTRL+Key_R);
  edit->setItemEnabled(prefsItem,FALSE);
  allowProps=FALSE;

  view = new QPopupMenu();
  CHECK_PTR (view);
  tBar=view->insertItem( "&ToolBar", this, SLOT(toggleToolBar()), CTRL+Key_M);
  view->setCheckable( TRUE );
  view->setItemChecked( tBar, TRUE );
  list=view->insertItem( "&Project List", this, SLOT(toggleList()), CTRL+Key_L);
  view->setItemChecked(list, TRUE);
  
  tools = new QPopupMenu();
  CHECK_PTR( tools );
  genItem=tools->insertItem( "&Generate", this, SLOT(gen()),CTRL+Key_G );
  tools->setItemEnabled(genItem,FALSE);
  allowGen=FALSE;
  srcItem=tools->insertItem("&Show Components",this,SLOT(classTree()),CTRL+Key_T);
  tools->setItemEnabled(srcItem,FALSE);
  allowTree=FALSE;
  tools->insertSeparator();
  userItem=tools->insertItem("&Preferences", this, SLOT(userPref()), CTRL+Key_P);

  QPopupMenu *elements=new QPopupMenu();
  CHECK_PTR( elements );
  elements->insertItem("Button",this, SLOT(setButton()));
  elements->insertItem("CheckBox",this, SLOT(setCheck_box()));
  elements->insertItem("Canvas",this, SLOT(setCanvas()));
  elements->insertItem("Choice",this, SLOT(setChoice()));
  elements->insertItem("Label",this, SLOT(setLabel()));
  elements->insertItem("List",this, SLOT(setList()));
  elements->insertItem("Menu",this, SLOT(setMenuItem()));
  elements->insertItem("MenuBar",this, SLOT(setMenuBar()));
  elements->insertItem("TextArea",this, SLOT(setTArea()));
  elements->insertItem("TextField",this, SLOT(setTField()));
  elements->insertItem("ScrollBar",this, SLOT(setScrollBar()));

  QPopupMenu *help = new QPopupMenu();
  CHECK_PTR( help );
  help->insertItem( "&About...", this, SLOT(about()), CTRL+Key_H );
  help->insertItem( "About &Qt...", this, SLOT(aboutQt()) );

  menu = new QMenuBar( this );
  CHECK_PTR( menu );

  menu->insertItem("&File", file );
  menu->insertItem("&Edit", edit );
  menu->insertItem("&View", view );
  menu->insertItem("&Components", elements);
  menu->insertItem("Too&ls",tools);
  menu->insertSeparator();
  menu->insertItem( "&Help", help );
  
  QString temp("BuildIT! v. ");
  temp+=VERSION;
  setCaption(temp.data());

  code=AR;
  clicked=FALSE;
  canUndo=FALSE;
  undoProps=FALSE;
  canSave=FALSE;
  canPaste=FALSE;
  canPrefs=FALSE;
  canClose=TRUE;

  PasteIT=FALSE;
  lastTouched=NULL;
  active=NULL;

  for(i=0;i<NUMPROJECTS;i++)
    state[i]=0;

  UserPrefs forload(this, "user-pref", homePath);  // added
  forload.loadConfig(options); // added

  pasteTree=new MyTree();
  CHECK_PTR(pasteTree);
  undoTree=new MyTree();
  CHECK_PTR(undoTree);

  progress=new QProgressBar(statusBar());
  CHECK_PTR( progress );
  progress->hide();
  statusBar()->message("BuildIT! is READY");

  int h=menu->height()+tool->height()+comps->height()+middle->height()+statusBar()->height();
  middle->move(0,menu->height()+tool->height()+comps->height()+20);  
  
  resize(WIDTH,h);
  setFixedWidth(WIDTH);
}

void MainWindow::prepare()
{
  int uid;
  struct passwd *info;

  QDir local("/usr/local/lib");

  if(!getenv("BIT_HOME"))   
    {
      if(local.exists("build/images")&&local.exists("build/templates"))
	bigPath=(QString("/usr/local/lib/build"));
      else
	{
	  QDir local("./");
	  if(local.exists("templates")&&local.exists("images"))
	    bigPath=QString("./");	
	  else
	    {
	      printf("\nBuildIT! ERROR: templates and images not found...\n");
	      exit();
	    }
	}
    }
  else
    bigPath=QString(getenv("BIT_HOME"));
  
  if(bigPath[bigPath.length()-1]!='/')
    bigPath+=QString("/");
  
  uid=getuid();
  info=getpwuid(uid);
  homePath=QString(info->pw_dir);
  if(homePath[homePath.length()-1]!='/')
    homePath+=QString("/");

  QDir home(homePath.data());
  
  if(home.exists(".build"))
    {
      if(! home.exists(".build/out"))
	home.mkdir(".build/out",FALSE);
      if(! home.exists(".build/tmp"))
	home.mkdir(".build/tmp",FALSE);
      else
	{
	  QDir remDir(homePath+".build/tmp");
	  QStrList *toRemove = new QStrList();
	  toRemove = remDir.entryList("*.tmp");
	  toRemove->first();
	  bool toEnd = FALSE;
	  if (toRemove->current() == NULL) toEnd = TRUE;
	  while (!toEnd)
	    {
	      remDir.remove(toRemove->current());
	      if (toRemove->next() == NULL) toEnd = TRUE;
	    }
	}
      if(! home.exists(".build/saves"))
	home.mkdir(".build/saves",FALSE); 
    }
  else
    {
      home.mkdir(".build",FALSE);
      home.mkdir(".build/out",FALSE);
      home.mkdir(".build/saves",FALSE);
      home.mkdir(".build/tmp",FALSE);
    }
}

void MainWindow::setToolBar()
{ 
  tool=new QToolBar("Project Operations",this,QMainWindow::Top,TRUE);
  CHECK_PTR (tool);

  //NEW BUTTON
  newBT=new QToolButton(QIconSet(QPixmap(bigPath+"images/new.xpm")),"New Project",0,this,SLOT(newOne()),tool,"new proj");
  CHECK_PTR (newBT);
  
  //OPEN BUTTON
  openBT=new QToolButton(QIconSet(QPixmap(bigPath+"images/open.xpm")),"Open Project",0,this,SLOT(open()),tool,"open proj");
  CHECK_PTR (openBT);

  //SAVE BUTTON
  saveBT=new QToolButton(QIconSet(QPixmap(bigPath+"images/save.xpm")),"Save Project",0,this,SLOT(save()),tool,"save proj");
  CHECK_PTR (saveBT);

  tool->addSeparator();

  //CUT BUTTON
  cutBT=new QToolButton(QIconSet(QPixmap(bigPath+"images/cut.xpm")),"Cut",0,this,SLOT(cut()),tool,"cut");
  CHECK_PTR (cutBT);

  //COPY BUTTON
  copyBT=new QToolButton(QIconSet(QPixmap(bigPath+"images/copy.xpm")),"Copy",0,this,SLOT(copy()),tool,"copy"); 
  CHECK_PTR (copyBT);

  //PASTE BUTTON
  pasteBT=new QToolButton(QIconSet(QPixmap(bigPath+"images/paste.xpm")),"Paste",0,this,SLOT(paste()),tool,"paste");
  CHECK_PTR (pasteBT);

  tool->addSeparator();

  //UNDO BUTTON
  undoBT=new QToolButton(QIconSet(QPixmap(bigPath+"images/undo.xpm")),"Undo",0,this,SLOT(undo()),tool,"undo");
  CHECK_PTR (undoBT);

  tool->addSeparator();

  //GENERATE BUTTON
  genBT=new QToolButton(QIconSet(QPixmap(bigPath+"images/generate.xpm")),"Generate Source",0,this,SLOT(gen()),tool,"generate");
  CHECK_PTR (genBT);

  //SOURCE BUTTON
  srcBT=new QToolButton(QIconSet(QPixmap(bigPath+"images/tree.xpm")),"Components Tree",0,this,SLOT(classTree()),tool,"tree");
  CHECK_PTR (srcBT);  

  //HELP BUTTON

  //operations
  (void)QWhatsThis::whatsThisButton( tool );
  QWhatsThis::add( newBT,newText, FALSE );
  QWhatsThis::add( openBT,openText, FALSE );
  QWhatsThis::add( saveBT,saveText, FALSE ); 
  QWhatsThis::add( cutBT,cutText, FALSE );
  QWhatsThis::add( copyBT,copyText, FALSE );
  QWhatsThis::add( pasteBT,pasteText, FALSE ); 
  QWhatsThis::add( undoBT,undoText, FALSE );
  QWhatsThis::add( genBT,genText, FALSE );
  QWhatsThis::add( srcBT,srcText, FALSE );

  QWhatsThis::add( middle,dlgText, FALSE );
  //components


  comps=new QToolBar("components",this,QMainWindow::Top,TRUE);
  CHECK_PTR (comps);

  QColor bg(darkGray);
  comps->setBackgroundColor(bg);

  bPB=new QToolButton(QIconSet(QPixmap(bigPath+"images/button.xpm")),"Button",0,this,SLOT(setButton()),comps,"button");
  bPB->setBackgroundColor(bg);
  CHECK_PTR (bPB);

  bCB=new QToolButton(QIconSet(QPixmap(bigPath+"images/checkbox.xpm")),"CheckBox",0,this,SLOT(setCheck_box()),comps,"checkbox");
  bCB->setBackgroundColor(bg); 
  CHECK_PTR (bCB);

  bCS=new QToolButton(QIconSet(QPixmap(bigPath+"images/canvas.xpm")),"Canvas",0,this,SLOT(setCanvas()),comps,"canvas");
  bCS->setBackgroundColor(bg);
  CHECK_PTR (bCS);

  bCO=new QToolButton(QIconSet(QPixmap(bigPath+"images/choice.xpm")),"Choice",0,this,SLOT(setChoice()),comps,"choice");
  bCO->setBackgroundColor(bg); 
  CHECK_PTR (bCO);  

  bLI=new QToolButton(QIconSet(QPixmap(bigPath+"images/list.xpm")),"List",0,this,SLOT(setList()),comps,"list");
  bLI->setBackgroundColor(bg); 
  CHECK_PTR (bLI);

  bTF=new QToolButton(QIconSet(QPixmap(bigPath+"images/text.xpm")),"Text Field",0,this,SLOT(setTField()),comps,"tfiled");
  bTF->setBackgroundColor(bg); 
  CHECK_PTR (bTF);

  bTA=new QToolButton(QIconSet(QPixmap(bigPath+"images/text_area.xpm")),"Text Area",0,this,SLOT(setTArea()),comps,"tarea");
  bTA->setBackgroundColor(bg); 
  CHECK_PTR (bTA);

  bSB=new QToolButton(QIconSet(QPixmap(bigPath+"images/scroll.xpm")),"Scrollbar",0,this,SLOT(setScrollBar()),comps,"scrollbar");
  bSB->setBackgroundColor(bg); 
  CHECK_PTR (bSB);

  bMB=new QToolButton(QIconSet(QPixmap(bigPath+"images/menu.xpm")),"Menu Bar",0,this,SLOT(setMenuBar()),comps,"menubar");
  bMB->setBackgroundColor(bg); 
  CHECK_PTR (bMB);

  bME=new QToolButton(QIconSet(QPixmap(bigPath+"images/menu_item.xpm")),"Menu Item",0,this,SLOT(setMenuItem()),comps,"menuitem");
  bME->setBackgroundColor(bg); 
  CHECK_PTR (bME);

  bLA=new QToolButton(QIconSet(QPixmap(bigPath+"images/label.xpm")),"Label",0,this,SLOT(setLabel()),comps,"label");
  bLA->setBackgroundColor(bg); 
  CHECK_PTR (bLA);

  QWhatsThis::add( bPB,PB_Text, FALSE );
  QWhatsThis::add( bCB,CB_Text, FALSE );
  QWhatsThis::add( bCS,CS_Text, FALSE );
  QWhatsThis::add( bCO,CO_Text, FALSE ); 
  QWhatsThis::add( bLI,LI_Text, FALSE );
  QWhatsThis::add( bTF,TF_Text, FALSE ); 
  QWhatsThis::add( bTA,TA_Text, FALSE );
  QWhatsThis::add( bSB,SB_Text, FALSE ); 
  QWhatsThis::add( bMB,MB_Text, FALSE );
  QWhatsThis::add( bME,ME_Text, FALSE );  
  QWhatsThis::add( bLA,LA_Text, FALSE );  
}

void MainWindow::newOne()
{
  int i,current;

  current=freeNum();
  if (current<0)
      QMessageBox::information( this, "warning","too many projects !");
  else 
    { 
      canPrefs=TRUE;canSave=TRUE;canClose=FALSE;
      edit->setItemEnabled(prefsItem,TRUE);
      allowProps=TRUE;
      file->setItemEnabled(saveItem,TRUE);
      file->setItemEnabled(saveAsItem,TRUE);
      allowSave=TRUE;
      file->setItemEnabled(closeItem,TRUE);
      allowClose=TRUE;
      tools->setItemEnabled(srcItem,TRUE);
      allowTree=TRUE;

      MyClass *tmp=new MyClass();
      CHECK_PTR (tmp);

      lastTouched=tmp;
      lastTouched->setTraced(tmp);
      lastTouched->counter++;
      
      if(!active)
	delete active;
      active=new Frame(tmp,"active");
      CHECK_PTR (active);

      setActive(tmp,FALSE);
      
      QString name=QString().setNum(lastTouched->counter)+QString(" RootClass")+QString().setNum(current);
      tmp->setName(name);
      tmp->setType(AP);

      tmp->setHomeDir(homePath);
      
      i=0;
      do
	{
	  QString check=homePath+QString(".build/saves/noname")+QString().setNum(i)+QString(".jbd");
	  i++;
	}
      while(!middle->isOpen(i));

      tmp->setFileName(homePath+QString(".build/saves/noname")+QString().setNum(current)+QString(".jbd"));

      QString *f1[NUMFEATURE1];  
      for (i = 0; i < NUMFEATURE1; i++) 
	f1[i] = new QString(30);

      int f2[NUMFEATURE2];
      for (i = 0; i < NUMFEATURE2; i++) 
	f2[i] = -1;

      *f1[0] = QString().setNum(lastTouched->counter);
      *f1[1] = name.copy();
      *f1[2] = "MyClass";
      *f1[3] = ""; 
     
      f2[0] = NUMFEATURE2;
      f2[1] = tmp->x();
      f2[2] = tmp->y();
      f2[3] = tmp->width();
      f2[4] = tmp->height();
      
      QColor col;
      col = tmp->backgroundColor();
      f2[5] = col.red();
      f2[6] = col.green();
      f2[7] = col.blue();
      col = tmp->foregroundColor();
      f2[8] = col.red();
      f2[9] = col.green();
      f2[10] = col.blue();
      
      QFont f;
      f = tmp->font();
      *f1[4] = f.family();
      f2[11] = f.pointSize();
      f2[12] = f.weight();
      if (f.italic())
	f2[13] = 1;
      else 
	f2[13] = 0;
      
      f2[14]=1;
      f2[20]=AP;

      tree[current].Add(f1, f2);
      tmp->which = current;
      middle->addProject(tmp);
      
      tmp->setMouseTracking(TRUE);
      connect(tmp,SIGNAL(press(QWidget *,QMouseEvent *)),SLOT(takePress(QWidget *,QMouseEvent *)));
      connect(tmp,SIGNAL(release(QWidget *,QMouseEvent *)),SLOT(takeRelease(QWidget *,QMouseEvent *)));
      connect(tmp,SIGNAL(itsMe(MyClass *)),SLOT(setTouched(MyClass *)));
      connect(tmp,SIGNAL(dbclick(QWidget *,QMouseEvent *)),SLOT(takeDbClick(QWidget *,QMouseEvent *)));
      connect(tmp,SIGNAL(mv(int,int)),SLOT(watchMouse(int,int)));
      connect(tmp,SIGNAL(leave()),SLOT(clear()));

      tmp->watchIt(tmp);
      tmp->show();
   
      for (i = 0; i < NUMFEATURE1; i++) 
	if (f1[i]) 
	  delete f1[i];
      
      state[current]=1;
    }
}

bool MainWindow::ShowWidget(MyTree *t, QWidget *parent,bool undo,bool first)
{
  int ch = t->GetNumChildren(),what,i;
  QWidget *next;
  QString name,widgetName;
  MyTree *temptree;
  
  if(!undo)
    lastTouched->counter++;
  
  name=*t->GetFeature1(1);
  widgetName=*t->GetFeature1(2);

  //only for undo
  if(widgetName==QString("MyClass"))
    {
      next=toUndo;
      what=AP;
    }

  //PUSHBUTTON  
  if(widgetName==QString("MyPushButton"))  
    {
      if(!undo)
	{
	  MyPushButton *tmp = new MyPushButton(t->GetFeature1(3)->data(), parent,name.data());
	  next=tmp;
	}
      else
	{
	  next=toUndo;
	  ((MyPushButton *)toUndo)->setText(t->GetFeature1(3)->data());
	  ((MyPushButton *)next)->evt.clear();
	}
      what=PB;
    }
  
  //RADIOBUTTON
  if (widgetName==QString("MyRadioButton"))
    {
      if(!undo)
	{
	  MyRadioButton *tmp = new MyRadioButton(t->GetFeature1(3)->data(), parent, name.data());
	  next = tmp;
	}
      else
	{
	  ((MyRadioButton *)toUndo)->setText(t->GetFeature1(3)->data());
	  next=toUndo;
	  ((MyRadioButton *)next)->evt.clear();
	}

      ((MyRadioButton *)next)->setGroup((t->GetFeature1(5))->data());
      if(t->GetFeature2(20)==1)
	((MyRadioButton *)next)->setChecked(TRUE);
      else
	((MyRadioButton *)next)->setChecked(FALSE);
      what=RB;
    }
      
  //CANVAS
  if (widgetName==QString("MyCanvas"))
    {
      if(!undo)
	{
	  MyCanvas *tmp = new MyCanvas(parent, name.data());
	  next=tmp;
	}
      else
	{
	  next=toUndo;
	  ((MyCanvas *)next)->evt.clear();
	}
      what=CS;
    }
  
  //CHOICE
  if (widgetName==QString("MyChoice"))
    {
      int i;

      if(!undo)
	{
	  MyChoice *tmp = new MyChoice(parent, name.data());
	  next = tmp;
	}
      else
	{
	  next=toUndo;
	  ((MyChoice *)next)->evt.clear();
	}

      ((MyChoice *)next)->clear();
      for (i=0; i<t->GetFeature2(20); i++)
	{
	  QString name = *(t->GetChildren(i))->GetFeature1(0);
	  temptree = t->Find(&name.copy());
	  ((MyChoice *)next)->insertItem(temptree->GetFeature1(3)->data());	
	}
      ((MyChoice *)next)->setCurrentItem(t->GetFeature2(21));
      what=CO;
    }

  //LIST
  if (widgetName==QString("MyList"))
    {
      int i;

      if(!undo)
	{
	  MyList *tmp = new MyList(parent, name.data());
	  next=tmp;
	}
      else
	{
	  next=toUndo;
	  ((MyList *)next)->evt.clear();
	}

      ((MyList *)next)->clear();
      for (i=0; i<t->GetFeature2(20); i++)
	{
	  QString name = *((t->GetChildren(i))->GetFeature1(0));
	  temptree = t->Find(&name.copy());
	  ((MyList *)next)->insertItem(temptree->GetFeature1(3)->data());
	  if(temptree->GetFeature2(20)==1)
	    ((MyList *)next)->setSelected(i,TRUE);
	}
      if(t->GetFeature2(21)==1)
	((MyList *)next)->setMulti(TRUE);
      else
	((MyList *)next)->setMulti(FALSE);
      what=LI;
    }

  //TEXTFIELD
  if (widgetName==QString("MyTextField"))
    {
      if(!undo)
	{
	  MyTextField *tmp = new MyTextField(parent, name.data());
	  next=tmp;
	}
      else
	{
	  next=toUndo;
	  ((MyTextField *)next)->evt.clear();
	}
      ((MyTextField *)next)->setText((t->GetFeature1(3))->data());
      ((MyTextField *)next)->setEcho((t->GetFeature1(5))->data());
      if(t->GetFeature2(20)==1)
	((MyTextField *)next)->setEdit(TRUE);
      else
	((MyTextField *)next)->setEdit(FALSE);
      what=TF;
    }

  //TEXTAREA
  if (widgetName==QString("MyTextArea"))
    {
      if(!undo)
	{
	  MyTextArea *tmp = new MyTextArea(parent, name.data());
	  next=tmp;
	}
      else
	{
	  next=toUndo;
	  ((MyTextArea *)next)->evt.clear();
	}

      if(t->GetFeature2(20)==1)
	((MyTextArea *)next)->setEdit(TRUE);
      else
	((MyTextArea *)next)->setEdit(FALSE);
      what=TA;
      ((MyTextArea *)next)->setScr(t->GetFeature2(21)); 
      QString txt;
      QString temp2=t->GetFeature1(1)->copy();
      
      QString temp1 = temp2.right(temp2.length()-temp2.find(" "));
      temp1=temp1.stripWhiteSpace();
      QString name = homePath+".build/tmp/" + temp1.copy() + ".tmp";

      QFile f2(name);
      ((MyTextArea *)next)->clear();
      if (f2.open(IO_ReadOnly) != FALSE)
	{
	  QTextStream t( &f2 );
	  QString s;          
	  while ( !t.eof() ) {
	    s = t.readLine(); 
	    ((MyTextArea *)next)->append((const char*) s);
	  }  
	  f2.close();
	}
    }

  //SCROLLBAR
  if (widgetName==QString("MyScrollBar"))
    {
      if(!undo)
	{
	  MyScrollBar *tmp = new MyScrollBar(parent, name.data());
	  next=tmp;
	}
      else
	{
	  next=toUndo;
	  ((MyScrollBar *)next)->evt.clear();
	}
      if((t->GetFeature2(20)==1)||(t->GetFeature2(20)==-1))
	((MyScrollBar *)next)->setOrientation(MyScrollBar::Horizontal);
      else
	((MyScrollBar *)next)->setOrientation(MyScrollBar::Vertical);
      ((MyScrollBar *)next)->setVal(t->GetFeature2(21));
      ((MyScrollBar *)next)->setMax(t->GetFeature2(22));
      ((MyScrollBar *)next)->setMin(t->GetFeature2(23));
      ((MyScrollBar *)next)->setPS(t->GetFeature2(24));
      what=SB;
    }

  //MENUBAR
  if (widgetName==QString("MyMenuBar"))
    {
      if(!undo)
	{
	  MyMenuBar *tmp = new MyMenuBar(parent, name.data());
	  next=tmp;
	}
      else
	{
	  next=toUndo;
	  ((MyMenuBar *)next)->evt.clear();
	}
      ((MyClass *)parent)->setMenu((MyMenuBar *)next,TRUE);
      ((MyMenuBar *)next)->setGeometry(0,0,parent->width(),20);
      what=MB;
    }

  //MENU
  if (widgetName==QString("MyMenu"))
    {
      MyMenuItem *it;
      int i;

      if(!undo)
	{
	  MyMenu *tmp = new MyMenu(t->GetFeature1(3)->data(),parent,name.data());
	  next=tmp;
	}
      else
	{
	  next=toUndo;
	  ((MyMenu *)next)->setText(t->GetFeature1(3)->data());
	  ((MyMenu *)next)->evt.clear();
	}

      // do it if it's a help menu
      if(t->GetFeature2(23)==1)
	((MyMenu *)next)->help=TRUE;
      else
	((MyMenu *)next)->help=FALSE;
      ((MyMenu *)next)->setShc(t->GetFeature2(20),t->GetFeature2(21),*t->GetFeature1(6));
      
      if(!undo)
	((MyMenuBar *)parent)->addMenu((MyMenu *)next);
      ((MyMenuBar *)parent)->adjust();

      ((MyMenu *)next)->items.clear();
      for (i=0; i<t->GetFeature2(22); i++)
	{
	  QString name = *((t->GetChildren(i))->GetFeature1(0));
	  temptree = t->Find(&name.copy());
	  it=new MyMenuItem(*temptree->GetFeature1(3),temptree->GetFeature2(24));
	  if(temptree->GetFeature2(22)==1)
	    {
	      it->setCB(TRUE);
	      if(temptree->GetFeature2(23)==1)
		it->setState(TRUE);
	      else
		it->setState(FALSE);
	    }
	  else
	      it->setCB(FALSE);
	  it->setKeys(*temptree->GetFeature1(6));
	  (((MyMenu *)next)->items).append(it);
	}
      what=ME;
    }
  
  //LABEL 
  if (widgetName==QString("MyLabel"))
    {
      if(!undo)
	{
	  MyLabel *tmp = new MyLabel(t->GetFeature1(3)->data(),parent, name.data());
	  next=tmp;
	}
      else
	{
	  next=toUndo;
	  ((MyLabel *)next)->evt.clear();
	}

      switch  (t->GetFeature2(20))
	{
	case 0:
	  ((MyLabel *)next)->setAlignment(AlignLeft|AlignVCenter);
	  break;
	case 1:
	  ((MyLabel *)next)->setAlignment(AlignCenter);
	  break;
	case 2:
	  ((MyLabel *)next)->setAlignment(AlignRight|AlignVCenter);	    
	  break;
	}
      what=LA;
    }
  // evets
  for(i=0;i<t->GetFeature2(19);i++)
    in(what,next,t->GetFeature1(10+i));
  
  //end of events
  if((what!=ME)&&(what!=MB))  //for menu don't set geometry & colors
   {
     next->setGeometry(t->GetFeature2(1), t->GetFeature2(2), t->GetFeature2(3), t->GetFeature2(4));  

     QColor bcol(t->GetFeature2(5), t->GetFeature2(6), t->GetFeature2(7));
     QColor fcol(t->GetFeature2(8), t->GetFeature2(9), t->GetFeature2(10));
     
     QPalette w_palette=next->palette();
     QColorGroup normal((w_palette.normal()).foreground(),
			bcol,
			(w_palette.normal()).light(),
			(w_palette.normal()).dark(),
			(w_palette.normal()).mid(),
			fcol,
			(w_palette.normal()).base());
     
     QPalette new_palette(normal,normal,normal);
     next->setPalette(new_palette);
   }
  if((first)&&(!undo))
    setUndo(next,ADD);

  //fonts section
  bool ital;
  if (t->GetFeature2(13) == 1) 
    ital = TRUE;
  else 
    ital = FALSE;
  QFont f((const char*) *t->GetFeature1(4), t->GetFeature2(11), t->GetFeature2(12), ital);
  next->setFont(f);
  //end of fonts section
           
  if(undo)
    {
      toUndo->setName(name.data());
      lastTouched->setTraced(next);
      return TRUE;      
    }
  else
    {
      next->setMouseTracking(TRUE);
      connect(next,SIGNAL(mv(QWidget *,QMouseEvent *)),SLOT(takeMove(QWidget *,QMouseEvent *)));
      connect(next,SIGNAL(press(QWidget *,QMouseEvent *)),SLOT(takePress(QWidget *,QMouseEvent *)));
      connect(next,SIGNAL(release(QWidget *,QMouseEvent *)),SLOT(takeRelease(QWidget *,QMouseEvent *)));      
      connect(next,SIGNAL(dbclick(QWidget *,QMouseEvent *)),SLOT(takeDbClick(QWidget *,QMouseEvent *)));
      connect(next,SIGNAL(leave()),SLOT(clear()));
      lastTouched->watchIt(next);      
      next->show(); 
    }

  lastTouched->setTraced(next);
  //something special for few classes :)
  setUndo(next,ADD);
  if ((what==CO)||(what==LI)||(what==ME))
    return TRUE;
  
  if (ch != 0) 
    {
      for (i = 0; i < ch; i++)
	ShowWidget(t->GetChildren(i), next,FALSE,FALSE);
      return TRUE;
    }
   else
     return TRUE;
}

void MainWindow::toggleToolBar()
{
  int h;    

  h=menu->height()+comps->height()+statusBar()->height();
  if(middle->isVisible())
    h+=middle->height();

  if ( tool->isVisible() ) 
    {
      tool->hide();
      view->setItemChecked( tBar, FALSE );
      middle->move(0,menu->height()+comps->height());
    } 
  else 
    {
      tool->show();
      view->setItemChecked( tBar, TRUE );
      middle->move(0,menu->height()+comps->height()+tool->height());
      h+=tool->height();
    }        
  resize(WIDTH,h+5);
  setFixedSize(WIDTH,h+5);
}

void MainWindow::toggleList()
{
  int h;

  h=menu->height()+comps->height()+statusBar()->height()+1;
  if(tool->isVisible())
    h+=tool->height();  
  if ( middle->isVisible() ) 
    {
      middle->hide();
      view->setItemChecked( list, FALSE );
    } 
  else 
    {
      middle->show();
      view->setItemChecked( list, TRUE);
      h+=middle->height();
    }  
  resize(WIDTH,h+5);
  setFixedSize(WIDTH,h+5);
}

void MainWindow::ShowReadTree(MyTree *t, QString s,int current)
{
  int i,ch;

  canPrefs=TRUE;
  edit->setItemEnabled(prefsItem,TRUE);
  allowProps=TRUE;
  edit->setItemEnabled(undoItem,FALSE);
  allowUndo=FALSE;
  edit->setItemEnabled(cutItem,FALSE);
  allowCut=FALSE;
  edit->setItemEnabled(copyItem,FALSE);
  allowCopy=FALSE;
  file->setItemEnabled(saveItem,TRUE);
  file->setItemEnabled(saveAsItem,TRUE);
  allowSave=TRUE;
  canSave=TRUE;
  file->setItemEnabled(closeItem,TRUE);
  allowClose=TRUE;
  tools->setItemEnabled(genItem,TRUE);
  allowGen=TRUE;
  tools->setItemEnabled(srcItem,TRUE);
  allowTree=TRUE;
 
  MyClass *tmp=new MyClass();
  CHECK_PTR (tmp);
	
  /* it is last touched */
  lastTouched=tmp;
  lastTouched->setTraced(tmp);

  tmp->counter = 1;

  /* set variable  name */
  tmp->setName(*t->GetFeature1(1));

  tmp->setHomeDir(homePath);
  tmp->setFileName(s.data());
  tmp->which=current;

  // set type frame aplet or window 
  tmp->setType(AP);
  // add to list
  middle->addProject(tmp);
  
  // set type and title 
  tmp->setType(t->GetFeature2(20));
  if(t->GetFeature2(20)==FR)
    tmp->setTitle(*t->GetFeature1(5));

  // fill events list
  for(i=0;i<t->GetFeature2(19);i++)
    in(AP,tmp,t->GetFeature1(10+i));

  tmp->setMouseTracking(TRUE);
  tmp->setGeometry(t->GetFeature2(1), t->GetFeature2(2), t->GetFeature2(3), t->GetFeature2(4));
  connect(tmp,SIGNAL(press(QWidget *,QMouseEvent *)),SLOT(takePress(QWidget *,QMouseEvent *)));
  connect(tmp,SIGNAL(release(QWidget *,QMouseEvent *)),SLOT(takeRelease(QWidget *,QMouseEvent *)));
  connect(tmp,SIGNAL(itsMe(MyClass *)),SLOT(setTouched(MyClass *)));
  connect(tmp,SIGNAL(dbclick(QWidget *,QMouseEvent *)),SLOT(takeDbClick(QWidget *,QMouseEvent *)));
  connect(tmp,SIGNAL(mv(int,int)),SLOT(watchMouse(int,int)));
  connect(tmp,SIGNAL(leave()),SLOT(clear()));
  tmp->watchIt(tmp);
  tmp->show();
  
  // colors & font
  QColor bcol(t->GetFeature2(5), t->GetFeature2(6), t->GetFeature2(7));
  QColor fcol(t->GetFeature2(8), t->GetFeature2(9), t->GetFeature2(10));
  
  QPalette w_palette=tmp->palette();
  QColorGroup normal((w_palette.normal()).foreground(),
		     bcol,
		     (w_palette.normal()).light(),
		     (w_palette.normal()).dark(),
		     (w_palette.normal()).mid(),
		     fcol,
		     (w_palette.normal()).base());
   
   QPalette new_palette(normal,normal,normal);
   tmp->setPalette(new_palette);

   ch = t->GetNumChildren();
   if (ch != 0)
     {
       tools->setItemEnabled(genItem,TRUE);
       allowGen=TRUE;
       for (i = 0; i < ch; i++)
	 ShowWidget(t->GetChildren(i), tmp);
     }
}

void MainWindow::open()
{
  int current;
  current=freeNum();
  if (current<0)
    statusBar()->message("TOO MANY PROJECTS !");
  else 
    { 
      QString tmp=homePath+QString(".build/saves");
      QString s(QFileDialog::getOpenFileName(tmp.data(),"*.jbd") );
      if(s.length()>0)
	{
	  if(middle->isOpen(s))
	    {
	      tree[current].File2Tree(s,homePath);
	      state[current]=1;
	      ShowReadTree(&tree[current],s,current);
	    }
	  else
	    statusBar()->message("THIS FILE IS ALREADY OPEN !");
	}
    }
}

void MainWindow::save()
{
  if(canSave)
    {
      canClose=TRUE;
      allowClose=TRUE;
      saveIt(lastTouched->getFileName(),FALSE);
    }
}

void MainWindow::saveAs()
{
  if(canSave)
    {
      canClose=TRUE;
      allowClose=TRUE;
      QString tmp=homePath+QString(".build/saves");

      QString s=QFileDialog::getSaveFileName(tmp.data(),"*jbd");
      if(s.length()==0)
	lastTouched->setFileName(lastTouched->getFileName());
      else{
	if (s.findRev(".jbd") == -1) 
	  s.append(".jbd");
	saveIt(s,TRUE);
      }
    }
}

void MainWindow::saveIt(QString name,bool val)
{
  if(val && QFile().exists(name.data()))
  {	 
    QString tmp=QString("File ")+name.right(name.length()-name.findRev("/")-1)+QString(" exists! Overwrite it ?");
    switch( QMessageBox::information( this, "warning",tmp.data(),"&Yes", "&No", "&Cancel",0,2 ) ) 
      {
      case 0:
	tree[whichLast()].SetFeature2(3, lastTouched->width());
	tree[whichLast()].SetFeature2(4, lastTouched->height());
	tree[whichLast()].Tree2File(name,homePath);
	lastTouched->setFileName(name);
	middle->changeProject(lastTouched);
	break;
      case 1:
	break;
      case 2:
	canClose=FALSE;
	break;
      }
  }
  else
    {
      tree[whichLast()].SetFeature2(3, lastTouched->width());
      tree[whichLast()].SetFeature2(4, lastTouched->height());
      tree[whichLast()].Tree2File(name,homePath);
      lastTouched->setFileName(name);
      middle->changeProject(lastTouched);
    }
  statusBar()->message(QString(QString("Wrote ")+lastTouched->getCpt()).data());
}

void MainWindow::closeIt()
{
  if(allowClose)
    {
      if(!canClose)
     	{
	  QString tmp=QString("Save ")+lastTouched->getFileName().right(lastTouched->getFileName().length()-lastTouched->getFileName().findRev("/")-1)+QString(" before closing ?\n");

	  switch( QMessageBox::information( this, "warning",tmp.data(),"&Yes", "&No", "&Cancel",0,2 ) )
	    {
	    case 0:
	      saveIt(lastTouched->getFileName(),FALSE);
	    case 1: 
	      state[whichLast()]=0;	      
	      tree[whichLast()].Remove(tree[whichLast()].GetFeature1(0));
	      lastTouched->hide();
	      middle->rmProject(lastTouched);
	      
	      
	      edit->setItemEnabled(undoItem,FALSE);canUndo=FALSE;
	      allowUndo=FALSE;
	      if(middle->getNum()==0)
		{ 
		  file->setItemEnabled(saveItem,FALSE);
		  file->setItemEnabled(saveAsItem,FALSE);
		  canSave=FALSE;
		  allowSave=FALSE;
		  file->setItemEnabled(closeItem,FALSE);
		  canClose=FALSE;
		  allowClose=FALSE;
		  tools->setItemEnabled(genItem,FALSE);
		  allowGen=FALSE;
		  tools->setItemEnabled(srcItem,FALSE);
		  allowTree=FALSE;
		  edit->setItemEnabled(prefsItem,FALSE);
		  allowProps=FALSE;
	     	}

	      edit->setItemEnabled(cutItem,FALSE);
	      allowCut=FALSE;
	      edit->setItemEnabled(copyItem,FALSE);
	      allowCopy=FALSE;
	      edit->setItemEnabled(pasteItem,FALSE);
	      allowPaste=FALSE;
	      break;
	    case 2: 
	      break;
	    }
      	} //canClose TRUE
      else
	{
	  state[whichLast()]=0;
	  tree[whichLast()].Remove(tree[whichLast()].GetFeature1(0));
	  lastTouched->hide();
	  middle->rmProject(lastTouched);
	  edit->setItemEnabled(undoItem,FALSE);canUndo=FALSE;
	  allowUndo=FALSE;
	  if(middle->getNum()<=0)
	    {
	      file->setItemEnabled(saveItem,FALSE);
	      file->setItemEnabled(saveAsItem,FALSE);
	      allowSave=FALSE;
	      canSave=FALSE;
	      file->setItemEnabled(closeItem,FALSE);
	      canClose=FALSE;
	      allowClose=FALSE;
	      tools->setItemEnabled(genItem,FALSE);
	      allowGen=FALSE;
	      tools->setItemEnabled(srcItem,FALSE);
	      allowTree=FALSE;
	      edit->setItemEnabled(prefsItem,FALSE);
	      allowProps=FALSE;
	    }
	  edit->setItemEnabled(cutItem,FALSE);
	  allowCut=FALSE;
	  edit->setItemEnabled(copyItem,FALSE);
	  allowCopy=FALSE;
	  edit->setItemEnabled(pasteItem,FALSE);
	  allowPaste=FALSE;
	}
    }
}
void MainWindow::userPref()
{
  tools->setItemEnabled(userItem, FALSE);
  UserPrefs *user = new UserPrefs(this, "userprefs", homePath);
  user->show(options);
}

void MainWindow::updatePref(int preftab[])
{
  for (int i=0; i<PREFNUMBER;i++)
      options[i] = preftab[i];
  tools->setItemEnabled(userItem, TRUE);

  //not active yet
  //updateView();
}

void MainWindow::updateView()
{
  bool tmp=FALSE;
  int h=menu->height()+comps->height()+statusBar()->height();
  
  if (!options[1])
    {
      if(middle->isVisible())
	{
	  tmp=TRUE;
	  middle->hide();
	  view->setItemChecked( list, FALSE );
	}
    }
  else 
    {
      if(!middle->isVisible())
	{
	  tmp=TRUE;
	  view->setItemChecked( list, TRUE);
	  middle->show();
	}
    }

  if (!options[0])
    {
      if(tool->isVisible())
	{
	  tmp=TRUE;
	  tool->hide();
	  view->setItemChecked(tBar,FALSE);
	  if(middle->isVisible())
	    middle->move(0,menu->height()+comps->height());
	}
    }
  else 
    {
      if(!tool->isVisible())
	{
	  tmp=TRUE;
	  view->setItemChecked( tBar, TRUE );
	  if(middle->isVisible())
	    middle->move(0,menu->height()+tool->height()+comps->height());
	  tool->show();
	}
    }

  if(tmp)
    {
      if(tool->isVisible())
	h+=tool->height();
      if(middle->isVisible())
	h+=middle->height();
      resize(WIDTH,h+5);
      setFixedSize(WIDTH,h+5);
    }
}

void MainWindow::classTree()
{
  if(allowTree)
    {
      ShowTree *show = new ShowTree(&tree[whichLast()],bigPath, options[2]);
      show->displayTree(TRUE);
    }
}
void MainWindow::gen()
{
  if(allowGen)
    {
      statusBar()->message("");
      progress->show();
      progress->setTotalSteps(tree[whichLast()].GetNumAllChildren());
      Generator s(progress,this, homePath, bigPath, options[3]);
      int result=s.generateSource(&tree[whichLast()]);
      
      QString temp1=tree[whichLast()].GetFeature1(1)->copy();
      temp1 = temp1.right(temp1.length()-temp1.findRev(" ")-1);
      
      QString name;
      name.sprintf(homePath+".build/out/%s.java", temp1.data());
      
      switch (tree[whichLast()].GetFeature2(20))
	{
	case AP:
	  if (result==1) 
	    {
	      s.finishSource(homePath, homePath+ ".build/tmp/applet.tmp", name);
	      s.genHtml(&tree[whichLast()]);
	    }
	  break;
	case FR:
	  if (result==1) s.finishSource(homePath, homePath+ ".build/tmp/frame.tmp", name); 
	  break;
	case WN:
	  if (result==1) s.finishSource(homePath, homePath+ ".build/tmp/window.tmp", name);
	}
      QString text;

      if (result!=-1) 
	{
	  if (!options[4])
	    {
	      s.removeComments(homePath+QString(".build/out/")+temp1+QString(".java"));
	      s.removeSpace(homePath+QString(".build/tmp/last.tmp"), homePath+QString(".build/out/")+temp1+QString(".java"));
	      s.copyFile(homePath+QString(".build/out/")+temp1+QString(".java"), homePath+QString(".build/tmp/last.tmp"));
	    }
	  text=QString("Your source file is: ~/.build/out/")+temp1+QString(".java");
	}
      else
	text="Generation failed";

      progress->hide();
      progress->reset();
      statusBar()->message(text.data());
    }
}


void MainWindow::exit()
{
  int j,i=middle->getNum();
  QDir remDir(homePath+".build/tmp");
  QStrList *toRemove = new QStrList();
  toRemove = remDir.entryList("*.tmp");
  toRemove->first();
  bool toEnd = FALSE;

  if(i>0)
    {
      QString text("Do you really want to exit?\nYou have ");
      if(i==1)
	text+=QString().setNum(i)+QString(" project still open !\n");
      else
	text+=QString().setNum(i)+QString(" projects still open !\n");
	
  
      switch( QMessageBox::information( this,"Attention!",text.data(),"&Yes", "&No", "&Cancel",0,2 ) ) 
	{
	case 0: 
	  qApp->quit();
	  middle->end();
	  
	  for(j=0;j<i;j++)
	    tree[j].Remove(tree[j].GetFeature1(0));

	  if (!toRemove->current()) 
	    toEnd = TRUE;
	  while (!toEnd)
	    {
	      remDir.remove(toRemove->current());
	      if (!toRemove->next()) 
		toEnd = TRUE;
	    }
	case 1: 
	  break;
	}
    }
  else
    {
      qApp->quit();	

      for(j=0;j<i;j++)
	tree[j].Remove(tree[j].GetFeature1(0));
 
      if (!toRemove->current()) 
	toEnd = TRUE;
      while (!toEnd)
	{
	  remDir.remove(toRemove->current());
	  if (!toRemove->next()) 
	    toEnd = TRUE;
	}
    }
}

void MainWindow::setUndo(QWidget *widget,int cmd)
{
  edit->setItemEnabled(undoItem,TRUE);
  allowUndo=TRUE;
  canClose=FALSE;

  undoID=QString(widget->name()).left(QString(widget->name()).find(" "));
  undoCMD=cmd;
  undoT=whichLast();
  switch (cmd)
    {
    case RM:
      toUndo=widget->parentWidget(); 
      break;
    case ADD:
      toUndo=widget;
      break;
    case MV:
      toUndo=widget;
      break;
    case CHANGE:
      QString id_tmp,id;
      int i,j,ch;

      toUndo=widget;
      undoParent=widget->parentWidget();

      //copy tree to pastetree
      QString nm=QString(toUndo->className());

      MyTree *tempTree;            
      QString *children;
      if((nm!=QString("MyChoice"))&&(nm!=QString("MyList"))&&(nm!=QString("MyMenu")))
	{  
	  //FLAT WIDGET
	  ch=1;
	  children=new QString[ch];
	  children[0]=undoID;
	}
      else
	{	  
	  tempTree=tree[whichLast()].Find(&undoID);
	  ch=tempTree->GetNumChildren();
	  ch++;
	  children=new QString[ch];
	  tempTree->getItems(children);
	}

      delete(undoTree);undoTree=NULL;
      undoTree=new MyTree();
      
      QString  *f1t[NUMFEATURE1];
      int       f2t[NUMFEATURE2];

      for (i=0; i<NUMFEATURE1; i++) 
	f1t[i]=new QString(30); 

      for(i=0;i<ch;i++)
	{
	  tempTree=tree[whichLast()].Find(&children[i]);
	  
	  for(j=0;j<NUMFEATURE1;j++)
	    *f1t[j]=tempTree->GetFeature1(j)->copy();
	  
	  *f1t[0]=children[i].replace(0,undoID.length(),QString("1"));
	  //because of move
	  for(j=0;j<NUMFEATURE2;j++)
	    f2t[j]=tempTree->GetFeature2(j);
	  if(i==0)
	    {
	      f2t[0]=NUMFEATURE2;
	      f2t[1]=undoX;
	      f2t[2]=undoY;
	      f2t[3]=undoW;
	      f2t[4]=undoH;
	    }
          undoTree->Add(f1t,f2t);
	}
      delete [] children;
      break;      
    }
}

void MainWindow::undo()
{
  if((canUndo) && (whichLast()==undoT))
    {
      MyTree *tempTree;

      switch(undoCMD)
	{
	case RM:
	  copyWidget(toUndo,0,0,TRUE);
	  break;	
	case ADD:
	  lastTouched->setTraced(toUndo);
	  cut();
	  break;
	case MV:
	  toUndo->setGeometry(undoX,undoY,undoW,undoH);
	  tempTree=tree[whichLast()].Find(&undoID);
	  tempTree->SetFeature2(1,undoX);
	  tempTree->SetFeature2(2,undoY);
	  tempTree->SetFeature2(3,undoW);
	  tempTree->SetFeature2(4,undoH);
	  setActive(toUndo,TRUE);
	  break;
	case CHANGE:
	  QString *f1t[NUMFEATURE1],id,id_tmp;
	  int f2t[NUMFEATURE2];
	  int i,j,ch;

	  undoID=QString(toUndo->name()).left(QString(toUndo->name()).find(" "));
	  
	  for (i=0; i<NUMFEATURE1; i++) 
	    f1t[i]=new QString(30);
	  
	  QString *children;
	  MyTree *tempTree;

	  if((QString(toUndo->className())!=QString("MyChoice"))&&(QString(toUndo->className())!=QString("MyList"))&&(QString(toUndo->className())!=QString("MyMenu")))
	    {  //FLAT WIDGET
	      ch=1;
	      children=new QString[ch];
	      children[0]=QString("1");
	    }
	  else
	    {	  
	      ch=undoTree->GetNumChildren();
	      ch++;
	      children=new QString[ch];
	      i=0;
	      undoTree->getItems(children);
	    }

	  for(i=0;i<ch;i++)
	    {
	      tempTree=undoTree->Find(&children[i]);

	      for(j=0;j<NUMFEATURE1;j++)
		*f1t[j]=tempTree->GetFeature1(j)->copy();

	      *f1t[0]=children[i].replace(0,1,undoID).copy();
	  

	      for(j=0;j<NUMFEATURE2;j++)
		f2t[j]=tempTree->GetFeature2(j);

	      if(ch>1)
		tree[whichLast()].Remove(f1t[0]);

	      if(tree[whichLast()].Find(f1t[0])!=NULL)
		tree[whichLast()].Update(f1t,f2t);
	      else
		tree[whichLast()].Add(f1t,f2t);
	    }
	  tempTree=tree[whichLast()].Find(&undoID);
	  ShowWidget(tempTree,undoParent,TRUE);

	  if(toUndo->className()!=QString("MyClass"))
	    setActive(toUndo,TRUE);

	  delete [] children;
	  break;
	}

      edit->setItemEnabled(undoItem,FALSE);
      delete(undoTree);undoTree=NULL;
      allowUndo=FALSE;
      canUndo=FALSE;
    }

}

void MainWindow::cut()
{
  QWidget *tmp;
  QString id_tmp,id;
  int i,j,ch;

  if(allowCut)
    {
      tmp=lastTouched->traced();
      if(QString(tmp->className())==QString("MyMenuBar"))
	lastTouched->setMenu(tmp,FALSE);
      setActive(tmp,FALSE);
      canUndo=TRUE;undoProps=FALSE;

      setUndo(tmp,RM);
      if(QString(tmp->className())==QString("MyMenu"))
	((MyMenuBar *)(lastTouched->mmb))->rmMenu((MyMenu *)tmp);    

      canPaste=TRUE;
      edit->setItemEnabled(pasteItem,TRUE);
      allowPaste=TRUE;

      id=QString(tmp->name()).left(QString(tmp->name()).find(" ")).copy();

      //copy removed tree to pastetree
      MyTree *tempTree=tree[whichLast()].Find(&id);
      ch=0;
      tempTree->countSub(tempTree,ch);

      QString *children=new QString[ch];

      i=0;
      tree[whichLast()].getSub(tempTree,children,i);
      
      delete(pasteTree);
      pasteTree=new MyTree();
      
      QString *f1t[NUMFEATURE1];
      int f2t[NUMFEATURE2];

      for (i=0; i<NUMFEATURE1; i++) 
	f1t[i]=new QString(30);

      for(i=0;i<ch;i++)
	{
	  tempTree=tree[whichLast()].Find(&children[i]);
	  
	  for(j=0;j<NUMFEATURE1;j++)
	    *f1t[j]=tempTree->GetFeature1(j)->copy();
	   
	  *f1t[0]=children[i].replace(0,id.length(),QString("1"));
	  
	  for(j=0;j<NUMFEATURE2;j++)
	    f2t[j]=tempTree->GetFeature2(j);
	  
          pasteTree->Add(f1t,f2t);
	}
      //remove component from main tree
      tree[whichLast()].Remove(&id);
      //cleaning
      delete [] children;
      delete(tmp);tmp=NULL;
    }
}

void MainWindow::copy()
{  
  QWidget *tmp;
  QString id_tmp,id;
  int i,j,ch;

  if(allowCopy)
    {
      tmp=lastTouched->traced();

      setActive(tmp,FALSE);
      canUndo=TRUE;undoProps=FALSE;canPaste=TRUE;
      edit->setItemEnabled(pasteItem,TRUE);
      allowPaste=TRUE;

      id=QString(tmp->name()).left(QString(tmp->name()).find(" ")).copy();

      //copy tree to pastetree
      MyTree *tempTree=tree[whichLast()].Find(&id);
      ch=0;
      tempTree->countSub(tempTree,ch);

      QString *children=new QString[ch];
      i=0;
      tree[whichLast()].getSub(tempTree,children,i); 
      
      delete(pasteTree);
      pasteTree=new MyTree();
      
      QString *f1t[NUMFEATURE1];
      int f2t[NUMFEATURE2];

      for (i=0; i<NUMFEATURE1; i++) 
	f1t[i]=new QString(30);
      
      for(i=0;i<ch;i++)
	{
	  tempTree=tree[whichLast()].Find(&children[i]);
	  
	  for(j=0;j<NUMFEATURE1;j++)
	    *f1t[j]=tempTree->GetFeature1(j)->copy();
	   
	  *f1t[0]=children[i].replace(0,id.length(),QString("1"));
	  
	  for(j=0;j<NUMFEATURE2;j++)
	    f2t[j]=tempTree->GetFeature2(j);
	  
          pasteTree->Add(f1t,f2t);
	}
      delete [] children;
    }
}

void MainWindow::paste()
{
  if(allowPaste)
    PasteIT=TRUE;
}

void MainWindow::prefs()
{
  if(allowProps)
    {
      canUndo=TRUE;undoProps=TRUE;
      setUndo(lastTouched->traced(),CHANGE);
      setActive(lastTouched->traced(),FALSE);
      Props *tmp=new Props(&tree[whichLast()],lastTouched->traced(),homePath);
    }
}
void MainWindow::about()
{
  QString temp("BuildIT!\nVersion ");
  temp+=VERSION;
  temp+="\n\nCopyright (c) 1998\n    Michal Switala <scisly@mat.uni.torun.pl>\n    Artur Prusakiewicz <prusak@mat.uni.torun.pl>\n\nhttp://www.mat.uni.torun.pl/~scisly/buildit/\n\nThis program is licenced under GNU General Public Licenece (GPL).\n";
  QMessageBox aboutwin(this, "BuildIT! help");
  aboutwin.setText(temp.data());
  //  aboutwin.setBackgroundColor(QColor(132,132,132));
  aboutwin.setIconPixmap(QPixmap(bigPath+"images/logohelp.gif"));
  aboutwin.show();
}

void MainWindow::aboutQt()
{
    QMessageBox::aboutQt( this, "Qt" );
}

void MainWindow::setActive(QWidget *widget,bool what,bool move)
{
 if(what)
   {
     active->move(widget->x()-1,widget->y()-1);
     active->resize(widget->width()+2,widget->height()+2);
     if(!move)
       {
	 active->raise();
	 widget->raise();
	 active->show();
       }
   }
 else
   active->hide();
}

void MainWindow::takeMove(QWidget *widget,QMouseEvent *event)
{
  int x,y,w,h,x1,y1,x2,y2,x3,y3;

  x=event->x();y=event->y();
  x1=widget->x();y1=widget->y();w=widget->width();h=widget->height();

  watchWidget(widget);

  if(clicked)
  {
    canUndo=TRUE;
    undoProps=TRUE;
    canClose=FALSE;
    setUndo(widget,MV);
    if(QString(widget->className()).contains("Menu")>0)
      place=0;
    switch(place)
      {
      case 1:widget->setGeometry(x1+x,y1+y,w-x,h-y);break;  // NW
      case 2:widget->setGeometry(x1+x,y1,w-x,h);break;      // W
      case 3:widget->setGeometry(x1+x,y1,w-x,y);break;      // SW
      case 4:widget->setGeometry(x1,y1,w,y);break;          // S
      case 5:widget->setGeometry(x1,y1,x,y);break;          // SE
      case 6:widget->setGeometry(x1,y1,x,h);break;          // E
      case 7:widget->setGeometry(x1,y1+y,x,h-y);break;      // NE
      case 8:widget->setGeometry(x1,y1+y,w,h-y);break;      // N
      case 9:widget->move(x1+x-clickX,y1+y-clickY);break;   // move
      case 10:widget->move(x1+x-clickX,y1);break;           // Menu
      }
    if(place>0)
      {
	widget->repaint(); 
	setActive(widget,TRUE,TRUE);
	
	QString temp=QString(widget->name()).left(QString(widget->name()).find(" ")); 
	
	MyTree *tmp; 
	tmp = tree[whichLast()].Find(&temp);
	tmp->SetFeature2(1, widget->x());
	tmp->SetFeature2(2, widget->y());
	tmp->SetFeature2(3, widget->width());
	tmp->SetFeature2(4, widget->height());
      }
  } 
  else
    {
      if((x<MR)&&(y<MR))  // polnocny zachod
	{
	  widget->setCursor(sizeAllCursor);
	  place=1;
	}
      else
	{
	  if((x<MR))  // zachod
	    {
	      widget->setCursor(sizeHorCursor);
	      place=2;
	    }
	  if(y<MR) // polnoc
	    {
	      widget->setCursor(sizeVerCursor);
	      place=8;
	    }
	}
      if((x<MR)&&(y>h-MR)) // poludniowy zachod
	{
	  widget->setCursor(sizeAllCursor);
	  place=3;
	}
      else
	{
	  if(y>h-MR) // polodnie
	    {
	      widget->setCursor(sizeVerCursor);
	      place=4;
	    }
	}
      if((y>h-MR)&&(x>w-MR)) //polodniowy wschod
	{
	  widget->setCursor(sizeAllCursor);
	  place=5;
	}
      else
	{
	  if(x>w-MR) //wschod
	    {
	      widget->setCursor(sizeHorCursor);
	      place=6;
	    } 
	}
      if((x>w-MR)&&(y<MR)) //  polnocny wschod
	{
	  widget->setCursor(sizeAllCursor);
	  place=7;
	}
      if((x>=MR)&&(y>=MR)&&(x<=w-MR)&&(y<=h-MR)) //wnetrze
	{
	 place=9;
	 widget->setCursor(arrowCursor);
	}
      if((x<MR)&&(y<MR))  // polnocny zachod
	{
	  widget->setCursor(sizeAllCursor);
	  place=1;
	}
      else
	{
	  if((x<MR))  // zachod
	    {
	      widget->setCursor(sizeHorCursor);
	      place=2;
	    }
	  if(y<MR) // polnoc
	    {
	      widget->setCursor(sizeVerCursor);
	      place=8;
	    }
	}
      if((x<MR)&&(y>h-MR)) // poludniowy zachod
	{
	  widget->setCursor(sizeAllCursor);
	  place=3;
	}
      else
	{
	  if(y>h-MR) // poludnie
	    {
	      widget->setCursor(sizeVerCursor);
	      place=4;
	    }
	}
      if((y>h-MR)&&(x>w-MR)) //poludniowy wschod
	{
	  widget->setCursor(sizeAllCursor);
	  place=5;
	}
      else
	{
	  if(x>w-MR) //wschod
	    {
	      widget->setCursor(sizeHorCursor);
	      place=6;
	    } 
	}
      if((x>w-MR)&&(y<MR)) //  polnocny wschod
	{
	  widget->setCursor(sizeAllCursor);
	  place=7;
	}
      if((x>=MR)&&(y>=MR)&&(x<=w-MR)&&(y<=h-MR)) //wnetrze
	{
	 place=9;
	 widget->setCursor(arrowCursor);
	}
    }
} // END OF takeMove

bool MainWindow::takePress(QWidget *widget,QMouseEvent *event)
{
  QString wname(widget->className());
  int x,y,x1,y1,w,h,x2,y2,x3,y3;
  x=event->x();y=event->y();
  
  canSave=TRUE;
  file->setItemEnabled(saveItem,TRUE);
  file->setItemEnabled(saveAsItem,TRUE);
  allowSave=TRUE;
  file->setItemEnabled(closeItem,TRUE);
  allowClose=TRUE;
  tools->setItemEnabled(genItem,TRUE);
  allowGen=TRUE;
  tools->setItemEnabled(srcItem,TRUE);
  allowTree=TRUE;
  //  file->setItemEnabled(srcItem,TRUE);
  edit->setItemEnabled(cutItem,TRUE);
  allowCut=TRUE;
  edit->setItemEnabled(copyItem,TRUE);
  allowCopy=TRUE;

  if(wname==QString("MyClass"))
     {
       edit->setItemEnabled(cutItem,FALSE);
       allowCut=FALSE;
       edit->setItemEnabled(copyItem,FALSE);
       allowCopy=FALSE;
     }
  
  if((wname!=QString("MyClass"))&&(wname!=QString("MyCanvas"))&&(wname!=QString("MyMenuBar")))
      code=AR;
  if((wname==QString("MyMenuBar"))&&(code!=ME))
    code=AR;
  if((code==MB)&&(wname!=QString("MyClass")))
    code=AR;
  
  if(event->button()==LeftButton)
    {
      if(PasteIT)
	{
	  copyWidget(widget,x,y);
	  return TRUE;
	}      
      if(code==AR)
	{
	  clicked=TRUE;
	  clickX=x;clickY=y;
	  undoX=widget->x();
	  undoY=widget->y();
	  undoW=widget->width();
	  undoH=widget->height();

	  watchWidget(widget);

	  if(active!=NULL)
	    delete active;

	  if(QString(widget->className())!=QString("MyClass"))
	    {
	      active=new Frame(widget->parentWidget(),"active");
	      setActive(widget,TRUE);
	    }
	  else
	    {
	      active=new Frame(widget,"active");
	      setActive(widget,FALSE);
	    }
	  widget->raise();
	}
      else  //if not an arrow
	{
	  if(lastTouched!=NULL)
	    lastTouched->counter++;
	 
	  
	  QString *tmpId=new QString(); 
	  tmpId->setNum(lastTouched->counter);
   
	  QString newId=QString(widget->name()).left(QString(widget->name()).find(" "))+QString("|")+*tmpId;

	  if(code==PB)
	    {
	      QString newVN(" push_button"+*tmpId);
	      MyPushButton *tmp=new MyPushButton("none",widget,newId+newVN);
	      tmp->move(x,y);
	      tmp->resize(60,20);
	      createWidget(tmp,newId);
	    }
	  if(code==RB)
	    { 
	      QString newVN(" radio_button"+*tmpId);
	      MyRadioButton *tmp=new MyRadioButton("none",widget,newId+newVN);
	      tmp->move(x,y);
	      tmp->resize(60,20);
	      createWidget(tmp,newId);
	    }
	  if(code==CS)
	    {
	      QString newVN(" canvas"+*tmpId);
	      MyCanvas *tmp=new MyCanvas(widget,newId+newVN);
	      tmp->move(x,y);
	      tmp->resize(300,200);
	      createWidget(tmp,newId);
	    }
	  if(code==CO)
	    {
	      QString newVN(" choice"+*tmpId);
	      MyChoice *tmp=new MyChoice(widget,newId+newVN);
	      tmp->move(x,y);
	      tmp->resize(60,20);
	      createWidget(tmp,newId);
	    }
	  if(code==LI)
	    {
	      QString newVN(" list"+*tmpId);
	      MyList *tmp=new MyList(widget,newId+newVN);
	      tmp->move(x,y);
	      tmp->resize(100,50);
	      createWidget(tmp,newId);
	    }	    
	  if(code==TF)
	    {
	      QString newVN(" text_field"+*tmpId);
	      MyTextField *tmp=new MyTextField(widget,newId+newVN);
	      tmp->move(x,y);
	      tmp->resize(80,20);
	      createWidget(tmp,newId);
	    }	    	  
	  if(code==TA)
	    {
	      QString newVN(" text_area"+*tmpId);
	      MyTextArea *tmp=new MyTextArea(widget,newId+newVN);
	      tmp->move(x,y);
	      tmp->resize(150,100);
	      createWidget(tmp,newId);
	    }	    	  	  
	  if(code==SB)
	    {
	      QString newVN(" scrollbar"+*tmpId);
	      MyScrollBar *tmp=new MyScrollBar(widget,newId+newVN);
	      tmp->move(x,y);
	      tmp->resize(100,10);
	      createWidget(tmp,newId);
	    }	    	  	  	  
	  if(code==MB)
	    {
	      if(!lastTouched->getMenu())
		{
		  QString newVN(" menubar");newVN+=*tmpId;
		  MyMenuBar *tmp=new MyMenuBar(widget,newId+newVN);
		  lastTouched->setMenu(tmp,TRUE);
		  tmp->move(0,0);
		  tmp->resize(lastTouched->width(),20);
		  createWidget(tmp,newId);
		}
	    }	    	  	  	  
	  if(code==ME)
	    {
	      if(QString(widget->className())==QString("MyMenuBar"))
		{
		  QString newVN(" menu"+*tmpId);
		  MyMenu *tmp=new MyMenu("none",widget,newId+newVN);
		  ((MyMenuBar *)widget)->addMenu(tmp);
		  createWidget(tmp,newId);
		}
	    }	    	  
	  if(code==LA)
	    {
	      QString newVN(" label"+*tmpId);
	      MyLabel *tmp=new MyLabel("none",widget,newId+newVN);
	      tmp->setAlignment(AlignCenter);
	      tmp->move(x,y);
	      tmp->resize(50,16);
	      createWidget(tmp,newId); 
	    }
	}
      code=AR;
    } //END of LeftButton
  else
    if(event->button()==RightButton)
      {
	if(active!=NULL)
	  delete active;
	if(QString(widget->className())!=QString("MyClass"))
	  {
	    active=new Frame(widget->parentWidget(),"active");
	    setActive(widget,TRUE);
	  }
	else
	  {
	    active=new Frame(widget,"active");
	    setActive(widget,FALSE);
	  }
	QPopupMenu *fastItem=new QPopupMenu();
	int paste;


	if(QString(widget->className())==QString("MyClass"))
	  {  
	    fastItem->move(lastTouched->x()+event->x(),lastTouched->y()+event->y()); 
	    fastItem->insertItem("Save",this,SLOT(save()));
	    fastItem->insertItem("Close",this,SLOT(closeIt()));
	    fastItem->insertSeparator();
	  }
	else
	  {
	    int tmpX=0,tmpY=0;
	    
	    fastItem->insertItem("Cut",this,SLOT(cut()));
	    fastItem->insertItem("Copy",this,SLOT(copy()));
	    QString tmp=QString(widget->name());
	    QString hisID=tmp.left(tmp.find(" "));
	    tree[whichLast()].getXY(hisID,tmpX,tmpY);
	    fastItem->move(lastTouched->x()+tmpX+widget->x()+widget->width()+15,lastTouched->y()+tmpY+widget->y()+event->y());
	  }
	paste=fastItem->insertItem("Paste",this,SLOT(paste()));
	fastItem->setItemEnabled(paste,canPaste);
	fastItem->insertSeparator();
	fastItem->insertItem("Properties...",this,SLOT(prefs()));

	fastItem->setFocus();
	fastItem->show();
      } 
  return TRUE;
}

void MainWindow::takeRelease(QWidget *widget,QMouseEvent *event)
{
  clicked=FALSE;clickX=0;clickY=0;
}

void MainWindow::takeDbClick(QWidget *widget,QMouseEvent *event)
{
  prefs();
}

void MainWindow::setTouched(MyClass *widget)
{
  if((widget!=lastTouched) &&(widget!=NULL) )
    {
      canPrefs=TRUE;canSave=TRUE;canClose=FALSE;
      allowSave=TRUE;
      allowProps=TRUE;
      lastTouched=widget;
    }
  middle->changeProject(lastTouched);
}

void MainWindow::createWidget(QWidget *widget,QString id)
{
  QString *f1[NUMFEATURE1];
  int f2[NUMFEATURE2],i;

  for(i=0;i<NUMFEATURE2;i++)
    f2[i]=-1;

  widget->setFont(QFont("courier",10));
 
  edit->setItemEnabled(cutItem,TRUE);
  allowCut=TRUE;
  edit->setItemEnabled(copyItem,TRUE);
  allowCopy=TRUE;
  canClose=FALSE;canUndo=TRUE;undoProps=FALSE;
  allowClose=TRUE;
  allowUndo=TRUE;
  allowProps=TRUE;
  edit->setItemEnabled(undoItem,TRUE);
  allowUndo=TRUE;

  setUndo(widget,ADD);
  
  PasteIT=FALSE;
  widget->setMouseTracking(TRUE);
  connect(widget,SIGNAL(mv(QWidget *,QMouseEvent *)),SLOT(takeMove(QWidget *,QMouseEvent *)));
  connect(widget,SIGNAL(press(QWidget *,QMouseEvent *)),SLOT(takePress(QWidget *,QMouseEvent *)));
  connect(widget,SIGNAL(release(QWidget *,QMouseEvent *)),SLOT(takeRelease(QWidget *,QMouseEvent *))); 
  connect(widget,SIGNAL(dbclick(QWidget *,QMouseEvent *)),SLOT(takeDbClick(QWidget *,QMouseEvent *)));
  connect(widget,SIGNAL(leave()),SLOT(clear()));

  lastTouched->watchIt(widget); //podlacz dotyk do appletu
  widget->show();           
  lastTouched->setTraced(widget);
      
  if(active!=NULL)
    delete active;

  active=new Frame(widget->parentWidget(),"active");
  setActive(widget,TRUE);
  // add to tree

  for (i = 0; i < NUMFEATURE1; i++) 
	f1[i] = new QString(30);

  *f1[0] = id.copy();
  *f1[1] = widget->name();
  *f1[2] = widget->className();

  if((QString(widget->className())!=QString("MyTextField"))&&(QString(widget->className())!=QString("MyMenuBar")))
    *f1[3] = "none";
  else
    *f1[3]="";

  f2[0] = NUMFEATURE2;
  f2[1] = widget->x();
  f2[2] = widget->y();
  f2[3] = widget->width();
  f2[4] = widget->height();
  
  QColor col;
  col = widget->backgroundColor();
  f2[5] = col.red();
  f2[6] = col.green();
  f2[7] = col.blue();
  col = widget->foregroundColor();
  f2[8] = col.red();
  f2[9] = col.green();
  f2[10] = col.blue();
  
  QFont f;
  f = widget->font();
  *f1[4] = f.family();
  f2[11] = f.pointSize();
  f2[12] = f.weight();
  if (f.italic() == TRUE) 
    f2[13] = 1;
  else 
    f2[13] = 0;

  f2[14]=0;  //subtree hidden
  if(QString(widget->className())==QString("MyScrollBar"))
    f2[20]=1;  //special for scrollbar (horizontal setting)  
  tree[whichLast()].Add(f1, f2);

  for (i = 0; i < NUMFEATURE1; i++) 
    if (f1[i]) 
      delete f1[i];	     
}

void MainWindow::copyWidget(QWidget *widget,int x,int y,bool undo)
{
  int i,j,nn=0;
  QString *f1t[NUMFEATURE1],id_parent,id,id_tmp,newid,tmp,to_search,root;
  int ch,f2t[NUMFEATURE2];

  ch=0;
  pasteTree->countSub(pasteTree,ch);
  QString *children=new QString[ch];
  i=0;
  pasteTree->getSub(pasteTree,children,i);

  id_parent=QString(widget->name());
  id=id_parent.left(id_parent.find(" "));

  for (i=0; i<NUMFEATURE1; i++) 
    f1t[i]=new QString(30);
  MyTree *tempTree;
  
  int realHelp[40];
  for(i=0;i<40;i++)
    realHelp[i]=0;

  for(i=0;i<ch;i++)
    {
      id_tmp=QString("");
      newid=QString("");
      tempTree=pasteTree->Find(&children[i]);
      
      for(j=0;j<NUMFEATURE1;j++)
	*f1t[j]=tempTree->GetFeature1(j)->copy();
      
      int count=children[i].contains("|");
      for(j=0;j<count;j++)
	{
	  if(realHelp[j]==0)
	    {
	      lastTouched->counter++;
	      id_tmp+=QString("|")+QString().setNum(lastTouched->counter);
	      realHelp[j]=lastTouched->counter;
	    }
	  else
	    id_tmp+=QString("|")+QString().setNum(realHelp[j]);
	}
      lastTouched->counter++;
      newid+=id+id_tmp+QString("|")+QString().setNum(lastTouched->counter);
      realHelp[j]=lastTouched->counter;

      *f1t[0]=newid.copy();
      newid+=f1t[1]->right(f1t[1]->length()-f1t[1]->find(" "));
      *f1t[1]=newid.copy();

      for(j=0;j<NUMFEATURE2;j++)
	f2t[j]=tempTree->GetFeature2(j);
      
      if(i==0)
	{
	  if(!undo)
	    {
	      f2t[1]=x;
	      f2t[2]=y;
	    }
	  root=f1t[0]->copy();
	}
      tree[whichLast()].Add(f1t,f2t);
    }
  PasteIT=FALSE;
  tempTree=tree[whichLast()].Find(&root);
  ShowWidget(tempTree,widget);
}

void MainWindow::watchMouse(int x,int y)
{
  QString *sentence=new QString();
  *sentence+="X="+QString().setNum(x)+" Y="+QString().setNum(y);
  statusBar()->message(sentence->data());
  delete(sentence);
}

void MainWindow::watchWidget(QWidget *widget)
{
  QString text=QString(widget->name()).right(QString(widget->name()).length()-QString(widget->name()).findRev(" "));

  text+=QString("  X=")+QString().setNum(widget->x())+QString(" Y=")+QString().setNum(widget->y())+QString(" W=")+QString().setNum(widget->width())+QString(" H=")+QString().setNum(widget->height());
  statusBar()->message(text.data());
}

int MainWindow::freeNum()
{
  int i;
  for(i=0;i<NUMPROJECTS;i++)
    if(state[i]==0)
      return i;
  return -1;
}

void MainWindow::in(int val,QWidget *wgt,QString *toPut)
{
  switch(val)
    {
    case AP:
    case WN:
    case FR:
      ((MyClass *)wgt)->evt.append(toPut);
      break;
    case PB:
       ((MyPushButton *)wgt)->evt.append(toPut);
      break;
    case RB:
       ((MyRadioButton *)wgt)->evt.append(toPut);
      break;
    case CO:
       ((MyChoice *)wgt)->evt.append(toPut);
      break;
    case CS:
       ((MyCanvas *)wgt)->evt.append(toPut);
      break;
    case LI:
       ((MyList *)wgt)->evt.append(toPut);
      break; 
    case TF:
       ((MyTextField *)wgt)->evt.append(toPut);
      break;       
    case TA:
       ((MyTextArea *)wgt)->evt.append(toPut);
      break;             
    case SB:
       ((MyScrollBar *)wgt)->evt.append(toPut);
      break;        
    case MB:
      ((MyMenuBar *)wgt)->evt.append(toPut);
      break;             
    case ME:
      ((MyMenu *)wgt)->evt.append(toPut);
      break;    
    case LA:
      ((MyLabel *)wgt)->evt.append(toPut);
      break;    
    default:
      printf("No such component\n");
    }
}

//Main

int main( int argc, char ** argv ,char **envp)
{
    QApplication a( argc, argv );

    if(argc>1)
      {
	printf("\n         GUI builder BuildIT! version 0.2\nCopyright 1998 Michal Switala & Artur Prusakiewicz\n\n");
	exit(0);
      }
  
    MainWindow m;
    a.setMainWidget( &m );
    m.show();
    return a.exec();
}















